@@ -1,6 +1,6 @@
 ;;; lsp-cmake.el --- description -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2020 emacs-lsp maintainers
+;; Copyright (C) 2020-2023 emacs-lsp maintainers
 
 ;; Author: emacs-lsp maintainers
 ;; Keywords: lsp, cmake
@@ -31,11 +31,30 @@
   :group 'lsp-mode
   :link '(url-link "https://github.com/regen100/cmake-language-server"))
 
+(defcustom lsp-cmake-server-command "cmake-language-server"
+  "The binary (or full path to binary) which executes the server."
+  :type 'string
+  :group 'lsp-cmake
+  :package-version '(lsp-mode . "9.0.0"))
+
+(defun lsp-cmake--download-server (_client callback error-callback update?)
+  "Install/update CMake language server using `pip
+
+Will invoke CALLBACK or ERROR-CALLBACK based on result.
+Will update if UPDATE? is t."
+  (lsp-async-start-process
+   callback
+   error-callback
+   "pip" "install" "cmake-language-server" (when update? "-U")))
+
 (lsp-register-client
- (make-lsp-client :new-connection (lsp-stdio-connection "cmake-language-server")
-                  :major-modes '(cmake-mode)
+ (make-lsp-client :new-connection (lsp-stdio-connection
+                                   (lambda ()
+                                     lsp-cmake-server-command))
+                  :activation-fn (lsp-activate-on "cmake")
                   :priority -1
-                  :server-id 'cmakels))
+                  :server-id 'cmakels
+                  :download-server-fn #'lsp-cmake--download-server))
 
 (lsp-consistency-check lsp-cmake)
 
