@@ -34,30 +34,37 @@ Version 1.34.3 minimum is required."
   :group 'lsp-mode
   :link '(url-link "https://github.com/OmniSharp/omnisharp-roslyn"))
 
+(defgroup lsp-csharp-omnisharp nil
+  "LSP support for C#, using the Omnisharp Language Server.
+Version 1.34.3 minimum is required."
+  :group 'lsp-mode
+  :link '(url-link "https://github.com/OmniSharp/omnisharp-roslyn")
+  :package-version '(lsp-mode . "9.0.0"))
+
 (defcustom lsp-csharp-server-install-dir
   (f-join lsp-server-install-dir "omnisharp-roslyn/")
   "Installation directory for OmniSharp Roslyn server."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
   :type 'directory)
 
 (defcustom lsp-csharp-server-path
   nil
   "The path to the OmniSharp Roslyn language-server binary.
 Set this if you have the binary installed or have it built yourself."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
   :type '(string :tag "Single string value or nil"))
 
 (defcustom lsp-csharp-test-run-buffer-name
   "*lsp-csharp test run*"
-  "The name of buffer used for outputing lsp-csharp test run results."
-  :group 'lsp-csharp
+  "The name of buffer used for outputting lsp-csharp test run results."
+  :group 'lsp-csharp-omnisharp
   :type 'string)
 
 (defcustom lsp-csharp-solution-file
   nil
   "Solution to load when starting the server.
 Usually this is to be set in your .dir-locals.el on the project root directory."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
   :type 'string)
 
 (defcustom lsp-csharp-omnisharp-roslyn-download-url
@@ -72,72 +79,65 @@ Usually this is to be set in your .dir-locals.el on the project root directory."
                    "omnisharp-win-x86.zip"))
 
                 ((eq system-type 'darwin)
-                 "omnisharp-osx.zip")
+                 (if (string-match "aarch64-.*" system-configuration)
+                     "omnisharp-osx-arm64-net6.0.zip"
+                   "omnisharp-osx-x64-net6.0.zip"))
 
                 ((and (eq system-type 'gnu/linux)
                       (or (eq (string-match "^x86_64" system-configuration) 0)
                           (eq (string-match "^i[3-6]86" system-configuration) 0)))
-                 "omnisharp-linux-x64.zip")
+                 "omnisharp-linux-x64-net6.0.zip")
 
                 (t "omnisharp-mono.zip")))
   "Automatic download url for omnisharp-roslyn."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
   :type 'string)
 
 (defcustom lsp-csharp-omnisharp-roslyn-store-path
   (f-join lsp-csharp-server-install-dir "latest" "omnisharp-roslyn.zip")
   "The path where omnisharp-roslyn .zip archive will be stored."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
+  :type 'file)
+
+(defcustom lsp-csharp-omnisharp-roslyn-binary-path
+  (f-join lsp-csharp-server-install-dir "latest" (if (eq system-type 'windows-nt)
+                                                     "OmniSharp.exe"
+                                                   "OmniSharp"))
+  "The path where omnisharp-roslyn binary after will be stored."
+  :group 'lsp-csharp-omnisharp
   :type 'file)
 
 (defcustom lsp-csharp-omnisharp-roslyn-server-dir
   (f-join lsp-csharp-server-install-dir "latest" "omnisharp-roslyn")
   "The path where omnisharp-roslyn .zip archive will be extracted."
-  :group 'lsp-csharp
+  :group 'lsp-csharp-omnisharp
   :type 'file)
 
 (lsp-dependency
  'omnisharp-roslyn
  `(:download :url lsp-csharp-omnisharp-roslyn-download-url
-             :store-path lsp-csharp-omnisharp-roslyn-store-path))
+             :decompress :zip
+             :store-path lsp-csharp-omnisharp-roslyn-store-path
+             :binary-path lsp-csharp-omnisharp-roslyn-binary-path
+             :set-executable? t)
+ '(:system "OmniSharp"))
 
-(defun lsp-csharp--download-server (_client callback error-callback _update?)
+(defun lsp-csharp--omnisharp-download-server (_client callback error-callback _update?)
   "Download zip package for omnisharp-roslyn and install it.
 Will invoke CALLBACK on success, ERROR-CALLBACK on error."
-  (lsp-package-ensure
-   'omnisharp-roslyn
-   (lambda ()
-     (lsp-unzip lsp-csharp-omnisharp-roslyn-store-path
-                lsp-csharp-omnisharp-roslyn-server-dir)
-     (unless (eq system-type 'windows-nt)
-       (let ((run-script (f-join lsp-csharp-omnisharp-roslyn-server-dir "run")))
-         (when (not (f-exists-p run-script))
-           ; create the `run' script when missing (e.g. when server binaries are extracted from omnisharp-mono.zip)
-           ; NOTE: we do not check for presence or version of mono in the system
-           (with-temp-file run-script
-             (insert "#!/bin/bash\n")
-             (insert "BASEDIR=$(dirname \"$0\")\n")
-             (insert "exec mono $BASEDIR/OmniSharp.exe $@\n")))
-         (set-file-modes run-script #o755)))
-     (funcall callback))
-   error-callback))
+  (lsp-package-ensure 'omnisharp-roslyn callback error-callback))
 
 (defun lsp-csharp--language-server-path ()
   "Resolve path to use to start the server."
-  (if lsp-csharp-server-path
-      lsp-csharp-server-path
-    (let ((server-dir lsp-csharp-omnisharp-roslyn-server-dir))
-      (when (f-exists? server-dir)
-        (f-join server-dir (cond ((eq system-type 'windows-nt) "OmniSharp.exe")
-                                 (t "run")))))))
+  (let ((executable-name (if (eq system-type 'windows-nt)
+                             "OmniSharp.exe"
+                           "OmniSharp")))
+    (or (and lsp-csharp-server-path
+             (executable-find lsp-csharp-server-path))
+        (executable-find executable-name)
+        (lsp-package-path 'omnisharp-roslyn))))
 
-(defun lsp-csharp--language-server-command ()
-  "Resolves path and arguments to use to start the server."
-  (append
-   (list (lsp-csharp--language-server-path) "-lsp")
-   (when lsp-csharp-solution-file (list "-s" (expand-file-name lsp-csharp-solution-file)))))
-
-(lsp-defun lsp-csharp-open-project-file ()
+(defun lsp-csharp-open-project-file ()
   "Open corresponding project file  (.csproj) for the current file."
   (interactive)
   (-let* ((project-info-req (lsp-make-omnisharp-project-information-request :file-name (buffer-file-name)))
@@ -215,7 +215,7 @@ PRESENT-BUFFER will make the buffer be presented to the user."
       (erase-buffer)))
 
   (when present-buffer
-      (display-buffer lsp-csharp-test-run-buffer-name)))
+    (display-buffer lsp-csharp-test-run-buffer-name)))
 
 (defun lsp-csharp--start-tests (test-method-framework test-method-names)
   "Run test(s) identified by TEST-METHOD-NAMES using TEST-METHOD-FRAMEWORK."
@@ -284,12 +284,11 @@ PRESENT-BUFFER will make the buffer be presented to the user."
     (message "lsp-csharp: No test method(s) found to be ran previously on this workspace")))
 
 (lsp-defun lsp-csharp--handle-os-error (_workspace (&omnisharp:ErrorMessage :file-name :text))
-  "Handle the 'o#/error' (interop) notification by displaying a message with lsp-warn."
+  "Handle the `o#/error' (interop) notification displaying a message."
   (lsp-warn "%s: %s" file-name text))
 
 (lsp-defun lsp-csharp--handle-os-testmessage (_workspace (&omnisharp:TestMessageEvent :message))
-  "Handle the 'o#/testmessage and display test message on lsp-csharp
-test output buffer."
+  "Handle the `o#/testmessage and display test message on test output buffer."
   (lsp-csharp--test-message message))
 
 (lsp-defun lsp-csharp--handle-os-testcompleted (_workspace (&omnisharp:DotNetTestResult
@@ -299,7 +298,7 @@ test output buffer."
                                                             :error-stack-trace
                                                             :standard-output
                                                             :standard-error))
-  "Handle the 'o#/testcompleted' message from the server.
+  "Handle the `o#/testcompleted' message from the server.
 
 Will display the results of the test on the lsp-csharp test output buffer."
   (let ((passed (string-equal "passed" outcome)))
@@ -336,14 +335,19 @@ using the `textDocument/references' request."
     (message "No references found")))
 
 (lsp-register-client
- (make-lsp-client :new-connection (lsp-stdio-connection
-                                   #'lsp-csharp--language-server-command
-                                   (lambda ()
-                                     (when-let ((binary (lsp-csharp--language-server-path)))
-                                       (f-exists? binary))))
-
-                  :major-modes '(csharp-mode csharp-tree-sitter-mode)
-                  :server-id 'csharp
+ (make-lsp-client :new-connection
+                  (lsp-stdio-connection
+                   #'(lambda ()
+                       (append
+                        (list (lsp-csharp--language-server-path) "-lsp")
+                        (when lsp-csharp-solution-file
+                          (list "-s" (expand-file-name lsp-csharp-solution-file)))))
+                   #'(lambda ()
+                       (when-let ((binary (lsp-csharp--language-server-path)))
+                         (f-exists? binary))))
+                  :activation-fn (lsp-activate-on "csharp")
+                  :server-id 'omnisharp
+                  :priority -1
                   :action-handlers (ht ("omnisharp/client/findReferences" 'lsp-csharp--action-client-find-references))
                   :notification-handlers (ht ("o#/projectadded" 'ignore)
                                              ("o#/projectchanged" 'ignore)
@@ -356,8 +360,106 @@ using the `textDocument/references' request."
                                              ("o#/testmessage" 'lsp-csharp--handle-os-testmessage)
                                              ("o#/testcompleted" 'lsp-csharp--handle-os-testcompleted)
                                              ("o#/projectconfiguration" 'ignore)
-                                             ("o#/projectdiagnosticstatus" 'ignore))
-                  :download-server-fn #'lsp-csharp--download-server))
+                                             ("o#/projectdiagnosticstatus" 'ignore)
+                                             ("o#/backgrounddiagnosticstatus" 'ignore))
+                  :download-server-fn #'lsp-csharp--omnisharp-download-server))
+
+;;
+;; Alternative "csharp-ls" language server support
+;; see https://github.com/razzmatazz/csharp-language-server
+;;
+(lsp-defun lsp-csharp--cls-metadata-uri-handler (uri)
+  "Handle `csharp:/(metadata)' uri from csharp-ls server.
+
+`csharp/metadata' request is issued to retrieve metadata from the server.
+A cache file is created on project root dir that stores this metadata and
+filename is returned so lsp-mode can display this file."
+
+  (-when-let* ((metadata-req (lsp-make-csharp-ls-c-sharp-metadata
+                              :text-document (lsp-make-text-document-identifier :uri uri)))
+               (metadata (lsp-request "csharp/metadata" metadata-req))
+               ((&csharp-ls:CSharpMetadataResponse :project-name
+                                                   :assembly-name
+                                                   :symbol-name
+                                                   :source) metadata)
+               (filename (f-join ".cache"
+                                 "lsp-csharp"
+                                 "metadata"
+                                 "projects" project-name
+                                 "assemblies" assembly-name
+                                 (concat symbol-name ".cs")))
+               (file-location (expand-file-name filename (lsp-workspace-root)))
+               (metadata-file-location (concat file-location ".metadata-uri"))
+               (path (f-dirname file-location)))
+
+    (unless (file-exists-p file-location)
+      (unless (file-directory-p path)
+        (make-directory path t))
+
+      (with-temp-file metadata-file-location
+        (insert uri))
+
+      (with-temp-file file-location
+        (insert source)))
+
+    file-location))
+
+(defun lsp-csharp--cls-before-file-open (_workspace)
+  "Set `lsp-buffer-uri' variable after C# file is open from *.metadata-uri file."
+
+  (let ((metadata-file-name (concat buffer-file-name ".metadata-uri")))
+    (setq-local lsp-buffer-uri
+                (when (file-exists-p metadata-file-name)
+                  (with-temp-buffer (insert-file-contents metadata-file-name)
+                                    (buffer-string))))))
+
+(defun lsp-csharp--cls-make-launch-cmd ()
+  "Return command line to invoke csharp-ls."
+
+  ;; emacs-28.1 on macOS has an issue
+  ;; that it launches processes using posix_spawn but does not reset sigmask properly
+  ;; thus causing dotnet runtime to lockup awaiting a SIGCHLD signal that never comes
+  ;; from subprocesses that quit
+  ;;
+  ;; as a workaround we will wrap csharp-ls invocation in "/bin/ksh -c" on macos
+  ;; so it launches with proper sigmask
+  ;;
+  ;; see https://lists.gnu.org/archive/html/emacs-devel/2022-02/msg00461.html
+
+  (let ((startup-wrapper (cond ((and (eq 'darwin system-type)
+                                     (version= "28.1" emacs-version))
+                                (list "/bin/ksh" "-c"))
+
+                               (t nil)))
+
+        (csharp-ls-exec (or (executable-find "csharp-ls")
+                            (f-join (or (getenv "USERPROFILE") (getenv "HOME"))
+                                    ".dotnet" "tools" "csharp-ls")))
+
+        (solution-file-params (when lsp-csharp-solution-file
+                                (list "-s" lsp-csharp-solution-file))))
+    (append startup-wrapper
+            (list csharp-ls-exec)
+            solution-file-params)))
+
+(defun lsp-csharp--cls-download-server (_client callback error-callback update?)
+  "Install/update csharp-ls language server using `dotnet tool'.
+
+Will invoke CALLBACK or ERROR-CALLBACK based on result.
+Will update if UPDATE? is t"
+  (lsp-async-start-process
+   callback
+   error-callback
+   "dotnet" "tool" (if update? "update" "install") "-g" "csharp-ls"))
+
+(lsp-register-client
+ (make-lsp-client :new-connection (lsp-stdio-connection #'lsp-csharp--cls-make-launch-cmd)
+                  :priority -2
+                  :server-id 'csharp-ls
+                  :activation-fn (lsp-activate-on "csharp")
+                  :before-file-open-fn #'lsp-csharp--cls-before-file-open
+                  :uri-handlers (ht ("csharp" #'lsp-csharp--cls-metadata-uri-handler))
+                  :download-server-fn #'lsp-csharp--cls-download-server))
 
 (lsp-consistency-check lsp-csharp)
 
