@@ -74,7 +74,7 @@
                                                          ,@lsp-clients-emmy-lua-args
                                                          ,lsp-clients-emmy-lua-jar-path)))
                                         #'lsp-clients-emmy-lua-test)
-  :major-modes '(lua-mode)
+  :activation-fn (lsp-activate-on "lua")
   :server-id 'emmy-lua
   :priority -1
   :notification-handlers (lsp-ht ("emmy/progressReport" #'ignore))))
@@ -96,12 +96,12 @@
 
 (defcustom lsp-clients-lua-language-server-bin
   (f-join lsp-clients-lua-language-server-install-dir
-          "extension/server/bin/"
+          "bin/"
           (pcase system-type
-            ('gnu/linux "Linux/lua-language-server")
-            ('darwin "macOS/lua-language-server")
-            ('windows-nt "Windows/lua-language-server.exe")
-            (_ "Linux/lua-language-server")))
+            ('gnu/linux "lua-language-server")
+            ('darwin "lua-language-server")
+            ('windows-nt "lua-language-server.exe")
+            (_ "lua-language-server")))
   "Location of Lua Language Server."
   :group 'lsp-lua-language-server
   :version "8.0.0"
@@ -110,7 +110,7 @@
 
 (defcustom lsp-clients-lua-language-server-main-location
   (f-join lsp-clients-lua-language-server-install-dir
-          "extension/server/main.lua")
+          "main.lua")
   "Location of Lua Language Server main.lua."
   :group 'lsp-lua-language-server
   :version "8.0.0"
@@ -337,7 +337,7 @@ tolerance for this setting. Please adjust it to the appropriate value."
   :group 'lsp-lua-language-server)
 
 (defcustom lsp-lua-runtime-file-encoding "utf8"
-  "File encoding. The 'ansi' option is only available under the 'Windows'
+  "File encoding.  The `ansi' option is only available under the `Windows'
 platform."
   :type '(choice (:tag "utf8" "ansi"))
   :package-version '(lsp-mode . "8.0.0")
@@ -482,6 +482,12 @@ and `../lib` ,exclude `../lib/temp`.
   :package-version '(lsp-mode . "8.0.0")
   :group 'lsp-lua-language-server)
 
+(defcustom lsp-lua-prefer-musl nil
+  "Whether or not to download the musl-compiled server."
+  :type 'boolean
+  :package-version '(lsp-mode . "8.0.0")
+  :group 'lsp-lua-language-server)
+
 (lsp-register-custom-settings
  '(("files.associations" lsp-lua-files-associations t)
    ("files.exclude" lsp-lua-files-exclude t)
@@ -530,19 +536,37 @@ and `../lib` ,exclude `../lib/temp`.
    ("Lua.completion.callSnippet" lsp-lua-completion-call-snippet)
    ("Lua.color.mode" lsp-lua-color-mode)))
 
-(defun lsp-lua-language-server-install (client callback error-callback update?)
+(defun lsp-lua-language-server-install-latest (client callback error-callback update?)
   "Download the latest version of lua-language-server and extract it to
 `lsp-lua-language-server-install-dir'."
   (ignore client update?)
-  (let ((store-path (expand-file-name "vs-lua" lsp-clients-lua-language-server-install-dir)))
+  (let ((store-path (expand-file-name "lua-language-server-github" lsp-clients-lua-language-server-install-dir)))
     (lsp-download-install
      (lambda (&rest _)
        (set-file-modes lsp-clients-lua-language-server-bin #o0700)
        (funcall callback))
      error-callback
-     :url (lsp-vscode-extension-url "sumneko" "lua" "1.17.4")
+     :url (lsp--find-latest-gh-release-url
+           "https://api.github.com/repos/sumneko/lua-language-server/releases/latest"
+           (format "%s%s.tar.gz"
+                   (pcase system-type
+                     ('gnu/linux
+                      (pcase (lsp-resolve-value lsp--system-arch)
+                        ('x64     "linux-x64")))
+                     ('darwin
+                      (pcase (lsp-resolve-value lsp--system-arch)
+                        ('x64     "darwin-x64")
+                        ('arm64   "darwin-arm64")))
+                     ('windows-nt
+                      (pcase (lsp-resolve-value lsp--system-arch)
+                        ('x64     "win32-x64")
+                        ('arm64   "win32-ia32")))
+                     (_
+                      (pcase (lsp-resolve-value lsp--system-arch)
+                        ('x64     "linux-x64"))))
+                   (if lsp-lua-prefer-musl "-musl" "")))
      :store-path store-path
-     :decompress :zip)))
+     :decompress (pcase system-type ('windows-nt :zip) (_ :targz)))))
 
 (lsp-register-client
  (make-lsp-client
@@ -551,11 +575,10 @@ and `../lib` ,exclude `../lib/temp`.
                                                          ,@lsp-clients-lua-language-server-args
                                                          ,lsp-clients-lua-language-server-main-location)))
                                         #'lsp-clients-lua-language-server-test)
-  :major-modes '(lua-mode)
+  :activation-fn (lsp-activate-on "lua")
   :priority -2
   :server-id 'lua-language-server
-  :download-server-fn #'lsp-lua-language-server-install))
-
+  :download-server-fn #'lsp-lua-language-server-install-latest))
 
 ;;; lua-lsp
 (defgroup lsp-lua-lsp nil
@@ -588,7 +611,7 @@ and `../lib` ,exclude `../lib/temp`.
                                           (or lsp-clients-lua-lsp-server-install-dir
                                               (f-join lsp-clients-luarocks-bin-dir "lua-lsp")))
                                         #'lsp-clients-lua-lsp-test)
-  :major-modes '(lua-mode)
+  :activation-fn (lsp-activate-on "lua")
   :priority -3
   :server-id 'lsp-lua-lsp))
 
@@ -630,7 +653,7 @@ and `../lib` ,exclude `../lib/temp`.
   :type 'file)
 
 (defcustom lsp-lua-roblox-server-download-url
-  (lsp-vscode-extension-url "Nightrains" "robloxlsp" "0.15.8")
+  (lsp-vscode-extension-url "Nightrains" "robloxlsp" "1.5.11")
   "Download url for Roblox Lua vscode extension."
   :group 'lsp-lua-roblox-language-server
   :version "8.0.0"
@@ -667,7 +690,7 @@ and `../lib` ,exclude `../lib/temp`.
                                                          ,@lsp-clients-lua-language-server-args
                                                          ,lsp-lua-roblox-language-server-main-location)))
                                         #'lsp-lua-roblox-language-server-test)
-  :major-modes '(lua-mode)
+  :activation-fn (lsp-activate-on "lua")
   :priority -4
   :server-id 'lua-roblox-language-server
   :download-server-fn #'lsp-lua-roblox-language-server-install))
