@@ -24,7 +24,7 @@
 (require 'lsp-mode)
 
 (defgroup lsp-completion nil
-  "LSP support for completion"
+  "LSP support for completion."
   :prefix "lsp-completion-"
   :group 'lsp-mode
   :tag "LSP Completion")
@@ -71,6 +71,12 @@ ignored."
   :type 'boolean
   :group 'lsp-completion)
 
+(defcustom lsp-completion-show-label-description t
+  "Whether or not to show description of completion candidates."
+  :type 'boolean
+  :group 'lsp-completion
+  :package-version '(lsp-mode . "9.0.0"))
+
 (defcustom lsp-completion-no-cache nil
   "Whether or not caching the returned completions from server."
   :type 'boolean
@@ -132,9 +138,19 @@ This will help minimize popup flickering issue in `company-mode'."
   "Dont do client-side reordering completion items when set.")
 
 (declare-function company-mode "ext:company")
-(declare-function company-doc-buffer "ext:company")
 (declare-function yas-expand-snippet "ext:yasnippet")
 
+(defun lsp-doc-buffer (&optional string)
+  "Return doc for STRING."
+  (with-current-buffer (get-buffer-create "*lsp-documentation*")
+    (erase-buffer)
+    (fundamental-mode)
+    (when string
+      (save-excursion
+        (insert string)
+        (visual-line-mode)))
+    (current-buffer)))
+
 (defun lsp-falsy? (val)
   "Non-nil if VAL is falsy."
   ;; https://developer.mozilla.org/en-US/docs/Glossary/Falsy
@@ -155,17 +171,20 @@ This will help minimize popup flickering issue in `company-mode'."
 
 (defun lsp-completion--annotate (item)
   "Annotate ITEM detail."
-  (-let (((&CompletionItem :detail? :kind?) (plist-get (text-properties-at 0 item)
+  (-let (((&CompletionItem :detail? :kind? :label-details?) (plist-get (text-properties-at 0 item)
                                                        'lsp-completion-item)))
     (concat (when (and lsp-completion-show-detail detail?)
               (concat " " (s-replace "\r" "" detail?)))
+            (when (and lsp-completion-show-label-description label-details?)
+              (when-let ((description (and label-details? (lsp:label-details-description label-details?))))
+                (format " %s" description)))
             (when lsp-completion-show-kind
               (when-let ((kind-name (and kind? (aref lsp-completion--item-kind kind?))))
                 (format " (%s)" kind-name))))))
 
 (defun lsp-completion--looking-back-trigger-characterp (trigger-characters)
-  "Return trigger character if text before point match any of the TRIGGER-CHARACTERS."
-  (unless (= (point) (point-at-bol))
+  "Return character if text before point match any of the TRIGGER-CHARACTERS."
+  (unless (= (point) (line-beginning-position))
     (seq-some
      (lambda (trigger-char)
        (and (equal (buffer-substring-no-properties (- (point) (length trigger-char)) (point))
@@ -253,35 +272,34 @@ The MARKERS and PREFIX value will be attached to each candidate."
   (lsp--while-no-input
     (->>
      (if items
-         (-->
-             (let (queries fuz-queries)
-               (-keep (-lambda ((cand &as &plist :label :start-point :score))
-                        (let* ((query (or (plist-get queries start-point)
-                                          (let ((s (buffer-substring-no-properties
-                                                    start-point (point))))
-                                            (setq queries (plist-put queries start-point s))
-                                            s)))
-                               (fuz-query (or (plist-get fuz-queries start-point)
-                                              (let ((s (lsp-completion--regex-fuz query)))
-                                                (setq fuz-queries
-                                                      (plist-put fuz-queries start-point s))
-                                                s)))
-                               (label-len (length label)))
-                          (when (string-match fuz-query label)
-                            (put-text-property 0 label-len 'match-data (match-data) label)
-                            (plist-put cand
-                                       :sort-score
-                                       (* (or (lsp-completion--fuz-score query label) 1e-05)
-                                          (or score 0.001)))
-                            cand)))
-                      items))
-           (if lsp-completion--no-reordering
-               it
-             (sort it (lambda (o1 o2)
-                        (> (plist-get o1 :sort-score)
-                           (plist-get o2 :sort-score)))))
-           ;; TODO: pass additional function to sort the candidates
-           (-map (-rpartial #'plist-get :item) it))
+         (--> (let (queries fuz-queries)
+                (-keep (-lambda ((cand &as &plist :label :start-point :score))
+                         (let* ((query (or (plist-get queries start-point)
+                                           (let ((s (buffer-substring-no-properties
+                                                     start-point (point))))
+                                             (setq queries (plist-put queries start-point s))
+                                             s)))
+                                (fuz-query (or (plist-get fuz-queries start-point)
+                                               (let ((s (lsp-completion--regex-fuz query)))
+                                                 (setq fuz-queries
+                                                       (plist-put fuz-queries start-point s))
+                                                 s)))
+                                (label-len (length label)))
+                           (when (string-match fuz-query label)
+                             (put-text-property 0 label-len 'match-data (match-data) label)
+                             (plist-put cand
+                                        :sort-score
+                                        (* (or (lsp-completion--fuz-score query label) 1e-05)
+                                           (or score 0.001)))
+                             cand)))
+                       items))
+              (if lsp-completion--no-reordering
+                  it
+                (sort it (lambda (o1 o2)
+                           (> (plist-get o1 :sort-score)
+                              (plist-get o2 :sort-score)))))
+              ;; TODO: pass additional function to sort the candidates
+              (-map (-rpartial #'plist-get :item) it))
        lsp-items)
      (-map (lambda (item) (lsp-completion--make-item item
                                                      :markers markers
@@ -319,39 +337,54 @@ The MARKERS and PREFIX value will be attached to each candidate."
   (alist-get (lsp:completion-item-kind? (get-text-property 0 'lsp-completion-item item))
              lsp-completion--kind->symbol))
 
+(defun lsp-completion--candidate-deprecated (item)
+  "Return if ITEM is deprecated."
+  (let ((completion-item (get-text-property 0 'lsp-completion-item item)))
+    (or (lsp:completion-item-deprecated? completion-item)
+        (seq-position (lsp:completion-item-tags? completion-item)
+                      lsp/completion-item-tag-deprecated))))
+
 (defun lsp-completion--company-match (candidate)
   "Return highlight of typed prefix inside CANDIDATE."
-  (let* ((prefix (downcase
-                  (buffer-substring-no-properties
-                   (plist-get (text-properties-at 0 candidate) 'lsp-completion-start-point)
-                   (point))))
-         (prefix-len (length prefix))
-         (prefix-pos 0)
-         (label (downcase candidate))
-         (label-len (length label))
-         (label-pos 0)
-         matches start)
-    (while (and (not matches)
-                (< prefix-pos prefix-len))
-      (while (and (< prefix-pos prefix-len)
-                  (< label-pos label-len))
-        (if (equal (aref prefix prefix-pos) (aref label label-pos))
-            (progn
-              (unless start (setq start label-pos))
-              (cl-incf prefix-pos))
-          (when start
-            (setq matches (nconc matches `((,start . ,label-pos))))
-            (setq start nil)))
-        (cl-incf label-pos))
-      (when start (setq matches (nconc matches `((,start . ,label-pos)))))
-      ;; Search again when the whole prefix is not matched
-      (when (< prefix-pos prefix-len)
-        (setq matches nil))
-      ;; Start search from next offset of prefix to find a match with label
-      (unless matches
-        (cl-incf prefix-pos)
-        (setq label-pos 0)))
-    matches))
+  (if-let ((md (cddr (plist-get (text-properties-at 0 candidate) 'match-data))))
+      (let (matches start end)
+        (while (progn (setq start (pop md) end (pop md))
+                      (and start end))
+          (setq matches (nconc matches `((,start . ,end)))))
+        matches)
+    (let* ((prefix (downcase
+                    (buffer-substring-no-properties
+                     ;; Put a safe guard to prevent staled cache from setting a wrong start point #4192
+                     (max (line-beginning-position)
+                          (plist-get (text-properties-at 0 candidate) 'lsp-completion-start-point))
+                     (point))))
+           (prefix-len (length prefix))
+           (prefix-pos 0)
+           (label (downcase candidate))
+           (label-len (length label))
+           (label-pos 0)
+           matches start)
+      (while (and (not matches)
+                  (< prefix-pos prefix-len))
+        (while (and (< prefix-pos prefix-len)
+                    (< label-pos label-len))
+          (if (equal (aref prefix prefix-pos) (aref label label-pos))
+              (progn
+                (unless start (setq start label-pos))
+                (cl-incf prefix-pos))
+            (when start
+              (setq matches (nconc matches `((,start . ,label-pos))))
+              (setq start nil)))
+          (cl-incf label-pos))
+        (when start (setq matches (nconc matches `((,start . ,label-pos)))))
+        ;; Search again when the whole prefix is not matched
+        (when (< prefix-pos prefix-len)
+          (setq matches nil))
+        ;; Start search from next offset of prefix to find a match with label
+        (unless matches
+          (cl-incf prefix-pos)
+          (setq label-pos 0)))
+      matches)))
 
 (defun lsp-completion--get-documentation (item)
   "Get doc comment for completion ITEM."
@@ -368,8 +401,8 @@ The MARKERS and PREFIX value will be attached to each candidate."
     (lsp:completion-item-documentation?)
     (lsp--render-element)))
 
-(defun lsp-completion--get-context (trigger-characters)
-  "Get completion context with provided TRIGGER-CHARACTERS."
+(defun lsp-completion--get-context (trigger-characters same-session?)
+  "Get completion context with provided TRIGGER-CHARACTERS and SAME-SESSION?."
   (let* ((triggered-by-char non-essential)
          (trigger-char (when triggered-by-char
                          (lsp-completion--looking-back-trigger-characterp
@@ -377,7 +410,8 @@ The MARKERS and PREFIX value will be attached to each candidate."
          (trigger-kind (cond
                         (trigger-char
                          lsp/completion-trigger-kind-trigger-character)
-                        ((equal (cl-second lsp-completion--cache) :incomplete)
+                        ((and same-session?
+                              (equal (cl-second lsp-completion--cache) :incomplete))
                          lsp/completion-trigger-kind-trigger-for-incomplete-completions)
                         (t lsp/completion-trigger-kind-invoked))))
     (apply #'lsp-make-completion-context
@@ -402,18 +436,9 @@ The MARKERS and PREFIX value will be attached to each candidate."
   (when (or (--some (lsp--client-completion-in-comments? (lsp--workspace-client it))
                     (lsp-workspaces))
             (not (nth 4 (syntax-ppss))))
-    (let* ((trigger-chars (->> (lsp--server-capabilities)
-                               (lsp:server-capabilities-completion-provider?)
-                               (lsp:completion-options-trigger-characters?)))
-           (bounds-start (or (-some--> (cl-first (bounds-of-thing-at-point 'symbol))
-                               (save-excursion
-                                 (ignore-errors
-                                   (goto-char (+ it 1))
-                                   (while (lsp-completion--looking-back-trigger-characterp
-                                           trigger-chars)
-                                     (cl-incf it)
-                                     (forward-char))
-                                   it)))
+    (let* ((trigger-chars (-> (lsp--capability-for-method "textDocument/completion")
+                              (lsp:completion-options-trigger-characters?)))
+           (bounds-start (or (cl-first (bounds-of-thing-at-point 'symbol))
                              (point)))
            result done?
            (candidates
@@ -440,7 +465,7 @@ The MARKERS and PREFIX value will be attached to each candidate."
                       (-let* ((resp (lsp-request-while-no-input
                                      "textDocument/completion"
                                      (plist-put (lsp--text-document-position-params)
-                                                :context (lsp-completion--get-context trigger-chars))))
+                                                :context (lsp-completion--get-context trigger-chars same-session?))))
                               (completed (and resp
                                               (not (and (lsp-completion-list? resp)
                                                         (lsp:completion-list-is-incomplete resp)))))
@@ -449,16 +474,16 @@ The MARKERS and PREFIX value will be attached to each candidate."
                                              ((lsp-completion-list? resp)
                                               (lsp:completion-list-items resp))
                                              (t resp))
-                                         (if (or completed
-                                                 (seq-some #'lsp:completion-item-sort-text? it))
-                                             (lsp-completion--sort-completions it)
-                                           it)
-                                         (-map (lambda (item)
-                                                 (lsp-put item
-                                                          :_emacsStartPoint
-                                                          (or (lsp-completion--guess-prefix item)
-                                                              bounds-start)))
-                                               it))))
+                                            (if (or completed
+                                                    (seq-some #'lsp:completion-item-sort-text? it))
+                                                (lsp-completion--sort-completions it)
+                                              it)
+                                            (-map (lambda (item)
+                                                    (lsp-put item
+                                                             :_emacsStartPoint
+                                                             (or (lsp-completion--guess-prefix item)
+                                                                 bounds-start)))
+                                                  it))))
                               (markers (list bounds-start (copy-marker (point) t)))
                               (prefix (buffer-substring-no-properties bounds-start (point)))
                               (lsp-completion--no-reordering (not lsp-completion-sort-initial-results)))
@@ -485,36 +510,32 @@ The MARKERS and PREFIX value will be attached to each candidate."
       (list
        bounds-start
        (point)
-       (lambda (probe _pred action)
+       (lambda (probe pred action)
          (cond
-          ;; metadata
-          ((equal action 'metadata)
-           `(metadata (category . lsp-capf)
-                      (display-sort-function . identity)))
-          ;; boundaries
-          ((equal (car-safe action) 'boundaries) nil)
-          ;; try-completion
-          ((null action)
-           (when-let ((cands (funcall candidates)))
-             (if (cl-rest cands) probe (cl-first cands))))
-          ;; test-completion: not return exact match so that the selection will
-          ;; always be shown
-          ((equal action 'lambda) nil)
-          ;; retrieve candidates
-          ((equal action t) (funcall candidates))))
+          ((eq action 'metadata)
+           '(metadata (category . lsp-capf)
+                      (display-sort-function . identity)
+                      (cycle-sort-function . identity)))
+          ((eq (car-safe action) 'boundaries) nil)
+          (t
+           (complete-with-action action (funcall candidates) probe pred))))
        :annotation-function #'lsp-completion--annotate
        :company-kind #'lsp-completion--candidate-kind
+       :company-deprecated #'lsp-completion--candidate-deprecated
        :company-require-match 'never
        :company-prefix-length
        (save-excursion
-         (goto-char bounds-start)
          (and (lsp-completion--looking-back-trigger-characterp trigger-chars) t))
        :company-match #'lsp-completion--company-match
-       :company-doc-buffer (-compose #'company-doc-buffer
+       :company-doc-buffer (-compose #'lsp-doc-buffer
                                      #'lsp-completion--get-documentation)
        :exit-function
        (-rpartial #'lsp-completion--exit-fn candidates)))))
 
+(defun lsp-completion--find-workspace (server-id)
+  (--first (eq (lsp--client-server-id (lsp--workspace-client it)) server-id)
+           (lsp-workspaces)))
+
 (defun lsp-completion--exit-fn (candidate _status &optional candidates)
   "Exit function of `completion-at-point'.
 CANDIDATE is the selected completion item.
@@ -527,11 +548,17 @@ Others: CANDIDATES"
               ((&plist 'lsp-completion-item item
                        'lsp-completion-start-point start-point
                        'lsp-completion-markers markers
+                       'lsp-completion-resolved resolved
                        'lsp-completion-prefix prefix)
                (text-properties-at 0 candidate))
               ((&CompletionItem? :label :insert-text? :text-edit? :insert-text-format?
                                  :additional-text-edits? :insert-text-mode? :command?)
-               item))
+               ;; see #3498 typescript-language-server does not provide the
+               ;; proper insertText without resolving.
+               (if (and (lsp-completion--find-workspace 'ts-ls)
+                        (not resolved))
+                   (lsp-completion--resolve item)
+                 item)))
         (cond
          (text-edit?
           (apply #'delete-region markers)
@@ -647,7 +674,7 @@ The return is nil or in range of (0, inf)."
                   "Update score variables given match range (A B)."
                   (setq score-numerator (+ score-numerator (- b a)))
                   (unless (= a len)
-                    ;; case mis-match will be pushed to near next rank
+                    ;; case mismatch will be pushed to near next rank
                     (unless (equal (aref query q-ind) (aref str a))
                       (cl-incf a 0.9))
                     (setq score-denominator
@@ -668,8 +695,8 @@ The return is nil or in range of (0, inf)."
       (/ score-numerator (1+ score-denominator) 1.0))))
 
 (defun lsp-completion--fix-resolve-data (item)
-  ;; patch `CompletionItem' for rust-analyzer otherwise resolve will fail
-  ;; see #2675
+  "Patch `CompletionItem' ITEM for rust-analyzer otherwise resolve will fail.
+See #2675"
   (let ((data (lsp:completion-item-data? item)))
     (when (lsp-member? data :import_for_trait_assoc_item)
       (unless (lsp-get data :import_for_trait_assoc_item)
@@ -681,7 +708,8 @@ The return is nil or in range of (0, inf)."
   (lsp-completion--fix-resolve-data item)
   (or (ignore-errors
         (when (lsp-feature? "completionItem/resolve")
-          (lsp-request "completionItem/resolve" item)))
+          (lsp-request "completionItem/resolve"
+                       (lsp-delete (lsp-copy item) :_emacsStartPoint))))
       item))
 
 (defun lsp-completion--resolve-async (item callback &optional cleanup-fn)
@@ -691,7 +719,8 @@ The CLEANUP-FN will be called to cleanup."
   (lsp-completion--fix-resolve-data item)
   (ignore-errors
     (if (lsp-feature? "completionItem/resolve")
-        (lsp-request-async "completionItem/resolve" item
+        (lsp-request-async "completionItem/resolve"
+                           (lsp-delete (lsp-copy item) :_emacsStartPoint)
                            (lambda (result)
                              (funcall callback result)
                              (when cleanup-fn (funcall cleanup-fn)))
@@ -715,6 +744,19 @@ The CLEANUP-FN will be called to cleanup."
   "Disable LSP completion support."
   (lsp-completion-mode -1))
 
+(defun lsp-completion-passthrough-all-completions (_string table pred _point)
+  "Passthrough all completions from TABLE with PRED."
+  (defvar completion-lazy-hilit-fn)
+  (when (bound-and-true-p completion-lazy-hilit)
+    (setq completion-lazy-hilit-fn
+          (lambda (candidate)
+            (->> candidate
+                 lsp-completion--company-match
+                 (mapc (-lambda ((start . end))
+                         (put-text-property start end 'face 'completions-common-part candidate))))
+            candidate)))
+  (all-completions "" table pred))
+
 ;;;###autoload
 (define-minor-mode lsp-completion-mode
   "Toggle LSP completion support."
@@ -729,10 +771,17 @@ The CLEANUP-FN will be called to cleanup."
                                (setq-local lsp-inhibit-lsp-hooks nil))))
     (cond
      (lsp-completion-mode
-      (setq-local completion-at-point-functions nil)
-      (add-hook 'completion-at-point-functions #'lsp-completion-at-point nil t)
-      (setq-local completion-category-defaults
-                  (add-to-list 'completion-category-defaults '(lsp-capf (styles basic))))
+      (make-local-variable 'completion-at-point-functions)
+      ;; Ensure that `lsp-completion-at-point' the first CAPF to be tried,
+      ;; unless user has put it elsewhere in the list by their own
+      (add-to-list 'completion-at-point-functions #'lsp-completion-at-point)
+      (make-local-variable 'completion-category-defaults)
+      (setf (alist-get 'lsp-capf completion-category-defaults) '((styles . (lsp-passthrough))))
+      (make-local-variable 'completion-styles-alist)
+      (setf (alist-get 'lsp-passthrough completion-styles-alist)
+            '(completion-basic-try-completion
+              lsp-completion-passthrough-all-completions
+              "Passthrough completion."))
 
       (cond
        ((equal lsp-completion-provider :none))
@@ -758,6 +807,8 @@ The CLEANUP-FN will be called to cleanup."
       (remove-hook 'completion-at-point-functions #'lsp-completion-at-point t)
       (setq-local completion-category-defaults
                   (cl-remove 'lsp-capf completion-category-defaults :key #'cl-first))
+      (setq-local completion-styles-alist
+                  (cl-remove 'lsp-passthrough completion-styles-alist :key #'cl-first))
       (remove-hook 'lsp-unconfigure-hook #'lsp-completion--disable t)
       (when (featurep 'company)
         (remove-hook 'company-completion-started-hook
