@@ -104,10 +104,9 @@ Results are meaningful only if FROM and TO are on the same line."
   (-doto (save-excursion
            (goto-char pos)
            (if (eq 'end-of-line lsp-lens-place-position)
-               (make-overlay (point-at-eol) -1 nil t t)
-             (make-overlay (point-at-bol) (1+ (point-at-eol)) nil t t)))
+               (make-overlay (line-end-position) -1 nil t t)
+             (make-overlay (line-beginning-position) (1+ (line-end-position)) nil t t)))
     (overlay-put 'lsp-lens t)
-    (overlay-put 'evaporate t)
     (overlay-put 'lsp-lens-position pos)))
 
 (defun lsp-lens--show (str pos metadata)
@@ -249,9 +248,11 @@ version."
                                                      :command?
                                                      :_pending pending))
   "Return t if LENS has to be loaded."
-  (and (< (window-start) (lsp--position-to-point start) (window-end))
-       (not command?)
-       (not pending)))
+  (let ((window (get-buffer-window (current-buffer))))
+    ;; (window-start/end) does not consider current window buffer if not passed manually
+    (and (< (window-start window) (lsp--position-to-point start) (window-end window))
+         (not command?)
+         (not pending))))
 
 (lsp-defun lsp--lens-backend-present? ((&CodeLens :range (&Range :start) :command?))
   "Return t if LENS has to be loaded."
@@ -309,12 +310,24 @@ CALLBACK - callback for the lenses."
           (funcall callback lsp-lens--backend-cache lsp--cur-version)
         (lsp-lens--backend-fetch-missing lsp-lens--backend-cache callback lsp--cur-version)))))
 
+(defun lsp-lens--refresh-buffer ()
+  "Trigger lens refresh on buffer."
+  (remove-hook 'lsp-on-idle-hook #'lsp-lens--refresh-buffer t)
+  (when (bound-and-true-p lsp-lens-mode)
+    (lsp-lens-refresh t)))
+
 (defun lsp--lens-on-refresh (workspace)
   "Clear lens within all buffers of WORKSPACE, refreshing all workspace buffers."
   (cl-assert (not (eq nil workspace)))
-  (cl-loop
-   for ws-buffer in (lsp--workspace-buffers workspace) do
-   (lsp-lens-refresh t ws-buffer)))
+  (->> (lsp--workspace-buffers workspace)
+       (mapc (lambda (buffer)
+               (lsp-with-current-buffer buffer
+                 (if (lsp--buffer-visible-p)
+                     (when (bound-and-true-p lsp-lens-mode)
+                       (lsp-lens-refresh t))
+                   (progn
+                     (add-hook 'lsp-on-idle-hook #'lsp-lens--refresh-buffer nil t)
+                     (lsp--idle-reschedule (current-buffer)))))))))
 
 ;;;###autoload
 (defun lsp-lens--enable ()
@@ -384,6 +397,7 @@ CALLBACK - callback for the lenses."
 (declare-function avy-process "ext:avy" (candidates &optional overlay-fn cleanup-fn))
 (declare-function avy--key-to-char "ext:avy" (c))
 (defvar avy-action)
+(defvar avy-style)
 
 ;;;###autoload
 (defun lsp-avy-lens ()
@@ -392,6 +406,10 @@ CALLBACK - callback for the lenses."
   (unless lsp-lens--overlays
     (user-error "No lenses in current buffer"))
   (let* ((avy-action 'identity)
+         (avy-style 'lsp-avy-lens)
+         (position (if (eq lsp-lens-place-position 'end-of-line)
+                       'after-string
+                     'before-string))
          (action (cl-third
                   (avy-process
                    (-mapcat
@@ -406,7 +424,7 @@ CALLBACK - callback for the lenses."
                      (let* ((path (mapcar #'avy--key-to-char path))
                             (str (propertize (string (car (last path)))
                                              'face 'avy-lead-face))
-                            (old-str (overlay-get ov 'before-string))
+                            (old-str (overlay-get ov position))
                             (old-str-tokens (s-split "|" old-str))
                             (old-token (seq-elt old-str-tokens index))
                             (tokens `(,@(-take index old-str-tokens)
@@ -416,14 +434,19 @@ CALLBACK - callback for the lenses."
                                          (concat str old-token))
                                       ,@(-drop (1+ index) old-str-tokens)))
                             (new-str (s-join (propertize "|" 'face 'lsp-lens-face) tokens))
-                            (new-str (if (s-ends-with? "\n" new-str)
+                            (new-str (if (or (s-ends-with? "\n" new-str)
+                                             (eq lsp-lens-place-position 'end-of-line))
                                          new-str
                                        (concat new-str "\n"))))
-                       (overlay-put ov 'before-string new-str)))
+                       (overlay-put ov position new-str)))
                    (lambda ()
-                     (--map (overlay-put it 'before-string
-                                         (overlay-get it 'lsp-original))
-                            lsp-lens--overlays))))))
+                     (--map
+                      (let ((original (overlay-get it 'lsp-original)))
+                        (overlay-put it position
+                                     (if (eq lsp-lens-place-position 'end-of-line)
+                                         (concat " " original)
+                                       original)))
+                      lsp-lens--overlays))))))
     (when action (funcall-interactively action))))
 
 (lsp-consistency-check lsp-lens)
